




--// TSLibrary_Misc.e
--// Tone koda <tone.skoda@siol.net>
--// Created on 30. September 2001.
--// Miscalenous general stuff.

--// Suggested namespace: misc





--// TSLibrary include files:
include TSLibrary_Debug.e as debug
include TSLibrary_Error.e
include TSLibrary_Types.e as types
--// Standard include files.
include CType.e
include Get.e
include Database.e
include wildcard.e





--//--//--//--//--//--//--//--//--//-- Variables and constants. --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//





--//--//--//--//--//--//--//--//--//-- Routines. --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//

--/*
-- file_exists [Created on 30. September 2001, 11:48]
-- The 'file_exists' function tests if file exists.
--
-- PARAMETERS
-- 'file_name'
--    Name of the file.
--
-- RETURN VALUES
-- If file exists, true.
--
-- If file doesn't exist, false.
--
--*/
global function file_exists (sequence file_name)
	integer a
	a = open (file_name, "r")
	if a = -1 then
		return false
	else
		close(a)
		return true
	end if	
end function

-- returns file name
-- example: "C:\dir\file.txt" returns "file.txt"
global function get_file_name (sequence fname)
	for i = length (fname) to 1 by -1 do
		if fname [i] = '\\' or fname [i] = '/' then
			return fname [i + 1 .. length (fname)]
		end if
	end for
	return fname
end function

-- returns file title
-- example: "C:\dir\file.txt" returns "file"
global function get_file_title (sequence fname)
	integer c -- current char
	integer dot_pos, i
	-- find last dot position
	i = length (fname)
	while 1 do
		if i < 1 then
			dot_pos = length (fname) + 1
			exit
		end if
		c = fname [i]
		if c = '.' then
			dot_pos = i
			exit
		elsif c = '\\' or c = '/' then
			dot_pos = length (fname) + 1
			exit
		end if
		i -= 1
	end while
	--find last slash position
	for j = dot_pos - 1 to 1 by -1 do
		c = fname [j]
		if c = '\\' or c = '/' then
			return fname [j + 1 .. dot_pos - 1]
		end if
	end for
	return fname [1 .. dot_pos - 1]
end function

-- returns file extension
-- example: "C:\dir\file.txt" returns "txt"
global function get_file_ext (sequence fname)
	for i = length (fname) to 1 by -1 do
		if fname [i] = '.' then
			return fname [i + 1 .. length (fname)]
		elsif fname [i] = '\\' or fname [i] = '/' then
			return ""
		end if
	end for
	return ""
end function

-- returns file directory
-- example: "C:\dir\file.txt" returns "C:\dir"
global function get_file_directory (sequence fname)
	integer c -- current char
	for i = length (fname) to 1 by -1 do
		c = fname [i]
		if c = '\\' or c = '/' then
			return fname [1 .. i - 1]
		elsif c = '.' then
			for j = i - 1 to 1 by -1 do
				c = fname [j]
				if c = '\\' or c = '/' then
					return fname [1 .. j - 1]
				end if
			end for
			return ""
		end if
	end for
	return fname
end function

--// Only "\\" as separator, not "/".
global function get_file_directory_fast (sequence fname)
    for i = length (fname) to 1 by -1 do
        if fname [i] = '\\' then
            return fname [1 .. i - 1]
        end if
    end for
    return fname --// "justword"
end function

--/*
-- remove_member [Created on 4. October 2001, 16:36]
-- The 'remove_member' function removes a member with index 'index' from sequence 's'.
--
-- PARAMETERS
-- 'index'
--    Index of the member to be removed.
--
-- 's'
--    Sequence from which to remove one member.
--
-- RETURN VALUES
-- Sequence 's' with one element removed.
--
-- REMRAKS
-- If index is out of range, the original sequence is returned. This functiion never fails.
--
--*/
global function remove_member (integer index, sequence s)
    if index < 1 or index > length (s) then --// Index is out of range.
        return s
    end if
    if index = length (s) then  --// Trying to remove last member from sequence.
        return s  [1 .. length (s) - 1]
    else                        --// Not trying to remove last member from sequence.
        return s  [1 .. index - 1] & s  [index + 1 .. length (s)]
    end if
end function

--/*
-- remove_last_member [Created on 23. November 2001, 18:32]
-- The 'remove_last_member' function removes last member of a sequence.
--
-- PARAMETERS
-- 's'
--    Sequence from which to remove last member.
--
-- RETURN VALUES
-- Sequence with removed last member.
-- If sequence was empty then empty sequence is returned.
--
--*/
global function remove_last_member (sequence s)
    if length (s) != 0 then --// Sequence is not empty.
        return s [1 .. length (s) - 1]
    else                    --// Sequence is empty.
        return {}
    end if
end function

--/*
-- insert_member [Created on 23. November 2001, 14:56]
-- The 'insert_member' function insert an object into sequence.
--
-- PARAMETERS
-- 'x'
--    This object is inserted.
-- 's'
--    In this sequence inserted.
-- 'after'
--    Index after which member in sequence to insert.
--
-- RETURN VALUES
-- Sequence with inserted object.
--
--*/
global function insert_member (object x, sequence s, integer after)
    if after >= length (s) then
        return append (s, x)
    else
        return append (s [1 .. after], x) & s [after + 1 .. length (s)]
    end if
end function

--// finds string in sequence with strings,  case is irrelavant
--// returns index where in s found or 0 if not found
global function findstr_nocase (sequence s, sequence str, integer from)
    sequence str_lower
    str_lower = lower (str)
    for i = from to length (s) do
	    if equal (str_lower, lower (s [i])) then
	        return i
	    end if
    end for
    return 0
end function

--/*
-- ensure_short_cut_front [Created on 7. October 2001, 01:35]
-- The 'ensure_short_cut_front' function ensures that sequence 's' is not longer that 'how_short'.
-- If it is longer than some excessive members are cut from the front.
-- PARAMETERS
-- 's'
--    Sequence to be checked for length and cut if necessary.
-- 'how_short'
--    How short should sequence be. If it is same long or shorter than this number it's ok.
--
-- RETURN VALUES
-- Cut or unchanged sequence.
--
--*/
global function ensure_short_cut_front (sequence s, integer how_short)
    if length (s) > how_short then --// Sequence is too big.
        return s[length (s) - how_short + 1 .. length (s)]
    end if
    return s
end function

--/*
-- sequence_to_string [Created on 8. October 2001, 22:06]
-- The 'sequence_to_string' function converts Euphoria sequence to string.
--
-- PARAMETERS
-- 's'
--    Sequence to be convereted to string.
--
-- RETURN VALUES
-- Sequence 's' converted to string. Curly braces mark sequences within sequences.
--
-- REMARKS
-- sequence {"string", 1, 2, 3, 4, {5, 6}, {}}
-- is converted to string:
-- "{{115, 116, 114, 105, 110, 103}, 1, 2, 3, 4, {5, 6}, {}}"
--
--*/
global function sequence_to_string (sequence s)
    --// One member of sequence.
    object member
    --// String returned.
    STRING string
    string = "{"
    --// Loop each member of sequence.
    for i = 1 to length (s) do
        member = s [i]
        if atom (member) then   --// Member is atom (number).
            string &= sprintf ("%d", member)
        else                    --// Member is sequence.
            string &= sequence_to_string (member)
        end if
        if i < length (s) then --// This is not last member, so add comma, to separate this member from next member.
            string &= ", "
        end if
    end for
    string &= "}"
    return string
end function

--/*
-- string_to_sequence [Created on 8. October 2001, 22:20]
-- The 'string_to_sequence' function converts string to Euphoria sequence.
-- Everywhere where there are curly braces {} in string, content inside them is considered as
-- one sequence.
--
-- PARAMETERS
-- 'string'
--    String to be converted to sequence.
--
-- RETURN VALUES
-- If success, euphoria sequence.
--
-- If failure, atom.
--
--*/
global function string_to_sequence (STRING string)
    --// Result of 'value ()'.
    sequence value_result
    value_result = value (string)
    if value_result [1] != GET_SUCCESS then --// Error.
        error ("Converting string \"" & string & "\" to sequence failed.")
        return 0
    else                                    --// Success.
        return value_result [2]
    end if
end function

--/*
-- is_letter [Created on 22. November 2001, 03:03]
-- The 'is_letter' function returns true if integer is a letter.
--
-- PARAMETERS
-- 'i'
--    Numer to test.
--
-- RETURN VALUES
-- True - It is letter.
--
-- False - It is not letter.
--
--*/
global function is_letter (atom a)
    if integer (a) = false or (a < ' ' or a > '}') then --// Not letter.
        return false
    else                                        --// It is letter!
        return true
    end if
end function

--/*
-- is_string [Created on 9. December 2001, 02:43]
-- The 'is_string' function checks if sequence is string.
--
-- PARAMETERS
-- 's'
--    Sequence to be checked.
--
-- RETURN VALUES
-- If it is string true is returned.
--
-- If it is not string false is returned.
--
--*/
global function is_string (object s)
    if not sequence (s) then
        return false
    end if
    for i = 1 to length (s) do
        if sequence (s [i]) = true or is_letter (s [i]) = false then --// Current member of 's' is not atom or letter.
            return false
        end if
    end for
    return true
end function

--/*
-- is_sequence_mixed_string [Created on 14. October 2001, 23:57]
-- The 'is_sequence_mixed_string' function finds out if sequence is a mixed string.
-- Mmixed string is a string which has both characters and non-characters inside itself.
-- It has no sequences.
--
-- PARAMETERS
-- 's'
--    Sequence to be checked.
--
-- RETURN VALUES
-- 0 if not string at all.
--
-- 1 if it is mixed sequence.
--
-- 2 if it is not mixed sequence, but pure string.
--
-- 3 if it is not mixed sequence, but there are only non-letters in it.
--
--*/
global function is_sequence_mixed_string (sequence s)
    --// One member of 's'.
    object member
    --// Was a character fonud inside sequence 's'?
    bool was_char_found
    --// Was non-character found inside sequence 's'?
    bool was_non_char_found
    was_char_found = false
    was_non_char_found = false
    --// Check each member of 's'.
    for i = 1 to length (s) do
        member = s [i]
        if atom (member) = false then    --// Sequence found inside sequence. This sequence is not string.
            return 0
        else                            --// Member is atom. Ok.
            if is_letter (member) = false then    --// Not character (letter).
                was_non_char_found = true
                if was_char_found = true then --// Both character and non-character were found inside sequence 's'.
                    --// Now just make sure till the end of the sequence 's' there are no sequences, but only atoms.
                    --// It doesn't matter if those atomes are number or letters.
                    for j = 1 to length (s) do
                        if sequence (s [j]) = true then --// Sequence found inside sequence. This sequence is not string.
                            return 0
                        end if
                    end for
                    --// Sequence 's' IS string!
                    return 1
                end if
            else                                    --// It is character (letter).
                was_char_found = true
                if was_non_char_found = true then --// Both character and non-character were found inside sequence 's'.
                    --// Now just make sure till the end of the sequence 's' there are no sequences, but only atoms.
                    --// It doesn't matter if those atomes are number or letters.
                    for j = 1 to length (s) do
                        if sequence (s [j]) = true then --// Sequence found inside sequence. This sequence is not string.
                            return 0
                        end if
                    end for
                    --// Sequence 's' IS string!
                    return 1
                end if
            end if
        end if
    end for
    --//
    --// A little debugging precaution:
    --//=>
        if Debug = true then --// In debug mode.
            if was_char_found = true and was_non_char_found = true then     --// both can't be true either.
                error ("Program error: Both character and non-character can't be found, because we are at end of the function.")
            end if
        end if
    if was_char_found = true then           --// Pure string.
        return 2
    elsif was_non_char_found = true then    --// Pure non-string.
        return 3
    elsif was_char_found = false and was_non_char_found = false then --// No atom at all was found. Sequence must be emptz.
        --// Let's consider emptz sequence as pure string, shall we? :)
        return 2
    end if
end function

--/*
-- db_open_and_check_errors [Created on 14. October 2001, 12:33]
-- The 'db_open_and_check_errors' procedure opens EDS file.
-- It is exactly same to 'db_open ()', except that it checks for errors and tells them to user when if happen.
--
-- PARAMETERS
-- 'file_name'
--    Name of the EDS file to open.
--*/
global procedure db_open_and_check_errors (STRING file_name)
    --// Success return code.
    integer success
    success = db_open (file_name, DB_LOCK_EXCLUSIVE)
    if success = DB_OPEN_FAIL then      --// Open failed
        error ("Couldn't open database file " & file_name & ".")
        return
    elsif success = DB_LOCK_FAIL then   --// Lock failed.
        error ("Couldn't open database file " & file_name & ". Lock failed.")
        return
    end if
end procedure

--/*
-- db_select_table_and_check_errors [Created on 16. October 2001, 18:50]
-- The 'db_select_table_and_check_errors' procedure selects table in EDS file.
-- It is exactly same to 'db_select_table ()', except that it checks for errors and tells them to user when if happen.
--
-- PARAMETERS
-- 'table_name'
--    .
--*/
global procedure db_select_table_and_check_errors (STRING table_name)
    if db_select_table (table_name) != DB_OK then --// Selecting table failed.
        error ("Couldn't find " & table_name & " table!")
    end if
end procedure

--/*
-- convert_mixed_string_to_pure [Created on 15. October 2001, 00:23]
-- The 'convert_mixed_string_to_pure' function converts mixed string to pure string.
-- Mimxed string is a string which has letters and numbers.
-- Pure string has only letters.
--
-- PARAMETERS
-- 'mixed_string'
--    Mixed string to be converted.
-- 'replacement_character'
--    Character to be put at place where non-character atoms are in 'mixed_string'.
--
-- RETURN VALUES
-- Converted 'pure' string.
--
--*/
global function convert_mixed_string_to_pure (sequence mixed_string, CHAR replacement_character)
    --// One member of 'mixed_string'.
    atom member
    --// This will be returned.
    STRING converted_string
    converted_string = repeat (0, length (mixed_string))
    --// Check each member of 'mixed_string'.
    for i = 1 to length (mixed_string) do
        member = mixed_string [i]
        if is_letter (member) = false then    --// Not character (letter).
            converted_string [i] = replacement_character
        else                                    --// It is letter.
            converted_string [i] = mixed_string [i]
        end if
    end for
    return converted_string
end function

-- finds x in s, which is assumed to be sorted
-- if x is not found returns the negative of the position where it would be
-- inserted (like EDS)
global function find_sorted(object x,sequence s)
    integer min,max,mid,test
    min=1
    max=length(s)
    while min<=max do
         mid=floor((min+max)/2)
         test=compare(x,s[mid])
         if test=0 then return mid end if
         if test>0 then
             min=mid+1
         else
             max=mid-1
         end if
   end while
   return -min
end function

--/*
-- get_sequence_member [Created on 29. November 2001, 18:55]
-- The 'get_sequence_member' function returns one member of a sequence.
-- Member is defined by a path.
--
-- PARAMETERS
-- 's'
--    This is sequence from which to get a member.
-- 'member_path'
--    Path to member. This is a sequence with just numbers.
--    It can be empty, then the whole sequence 's' is returned.
--    Example: if s = {1, {2, {3, 4}, 5}, 6, 7}
--             and member_path = {2}
--             {2, {3, 4} is returned.
--             , or member_path = {2, 2}
--             {3, 4} is returned.
--             , or member_path = {2, 2, 1}
--             3 is returned.
--             , or member_path = {2, 2, 2}
--             4 is returned.
--             , or member_path = {1}
--             1 is returned.
--             , or member_path = {2, 3}
--             5 is returned.
--             etc ...
--
-- RETURN VALUES
-- Euphoria object, a member of the sequence 's', or the whole sequence 's'.
--
--*/
global function get_sequence_member (sequence s, sequence member_path)
    --// Current sequence. It shrinks down till we get the member we want.
    sequence current_sequence
    if equal (member_path, {}) = true then --// Path to member is empty. Simply return whole sequence.
        return s
    end if
    current_sequence = s
    --// All members excpect last which we get by help of 'member_path' must be sequence.
    for i = 1 to length (member_path) - 1 do
        current_sequence = current_sequence [member_path [i]]
    end for
    --// Last member doesn't need to be sequence. We get it here and return it.
    return current_sequence [member_path [length (member_path)]]
end function

global function remove_last (sequence s)
    return s [1 .. length (s) - 1]
end function

global function print_format( object o )

    -- returns object formatted for wPrint
    sequence s

    if atom( o ) then
        -- number
		if o >= ' ' and o <= '}' then
			return sprintf( "%d'%s'", {o,o} )
			--return sprintf( "%s", o )
		else			
			return sprintf( "%d", o )
		end if
    else
        -- list
        s = "{"
        for i = 1 to length( o ) do
            s = s & print_format( o[i] )
            if i < length( o ) then
                s = s & ","
            end if
        end for
        s = s & "}"
        return s
    end if

end function

--/*
-- make_file_empty_if_it_exists [Created on 6. December 2001, 16:44]
-- The 'make_file_empty_if_it_exists' procedure empties file if it exists.
-- If it doesn't exists then nothing happens.
--
-- PARAMETERS
-- 'file_name'
--    Name of the file to be emptied.
--*/
global procedure make_file_empty_if_it_exists (STRING file_name)
    --// File number.
    integer fn
    fn = open (file_name, "w")
    if fn != -1 then --// File did exist.
        close (fn)
    end if
end procedure

--/*
-- get_number_of_digits [Created on 8. December 2001, 02:20]
-- The 'get_number_of_digits' function returns how many digits are there in
-- decimal number 'num'
--
-- PARAMETERS
-- 'num'
--    Decimal number for which to get number of digits.
--
-- RETURN VALUES
-- Number of digits in 'num'.
--
--*/
global function get_number_of_digits (integer num)
    return length (sprintf ("%d", num))
end function

--/*
-- string_to_number [Created on 12. December 2001, 00:00]
-- The 'string_to_number' function converts string to integer.
-- If it could not be converted error message box is displayed.
--
-- PARAMETERS
-- 'string'
--    String to convert.
--
-- RETURN VALUES
-- 'sstring' converted to integer.
--
--*/
global function string_to_number (STRING string)
    --// Result of 'value ()'.
    sequence val
    val = value (string)
    if val [1] != GET_SUCCESS then --// Value could not be gotten.
        error ("Couldn't get value for \"" & string & "\".")
        return 0
    end if
    return val [2]
end function

--/*
-- number_to_string [Created on 22. June 2002, 16:55]
-- The 'number_to_string' function converts number (atom or integer)
-- to string.
--
-- PARAMETERS
-- 'number'
--    .
--
-- RETURN VALUES
-- STRING.
--*/
global function number_to_string (atom number)
    if integer (number) then --// whole number
        return sprintf ("%d", number)
    else --// floating point number
        return sprintf ("%f", number)
    end if
end function

--/*
-- is_any_lowest_integer_inside [Created on 21. December 2001, 03:47]
-- The 'is_any_lowest_integer_inside' function finds is sequence has any
-- 'LOWEST_INTEGER' constant inside itself.
--
-- PARAMETERS
-- 's'
--    Sequence to be searched.
--
-- RETURN VALUES
-- If 'LOWEST_INTEGER' is inside sequence true is returned.
-- 
-- If 'LOWEST_INTEGER' is not inside sequence false is returned.
--*/
global function is_any_lowest_integer_inside (sequence s)
    for i = 1 to length (s) do
        if atom (s [i]) and s [i] = LOWEST_INTEGER then
            return true
        end if
    end for
    return false
end function

--/*
-- find_from_pos [Created on 27. November 2001, 15:45]
-- The 'find_from_pos' function find an object in sequence 
-- starting from a given position.
--
-- PARAMETERS
-- 'x'
--    This object finding.
-- 's'
--    Searching in this string.
-- 'from_pos'
--    At this position starting search (including).
-- 'direction'
--   In which direction to search.
--   1 = down.
--   - 1 = up.
--
-- RETURN VALUES
-- Position of 'x' is 's' if found.
--
-- 0 if not found.
--
-- WARNING: UNTESTED.
--
--*/
global function find_from_pos (object x, sequence s, integer from_pos, integer direction)
    --// Current position in 's'.
    integer current_pos
    current_pos = from_pos
    --// Loop until 'x' is found or we get to end of sequence 's'.
    while 1 do
        if current_pos > length (s) or current_pos < 1 then --// We have searched whole string.
            exit
        end if
        if s [current_pos] = x then
            return current_pos
        end if
        current_pos += direction
    end while
    return 0
end function

--/*
-- fast_file_read [Created on 21. December 2001, 14:25]
-- The 'fast_file_read' function reads whole file and returns string of file contents.
--
-- PARAMETERS
-- 'inFile'
--    Name of the file to read.
--
-- RETURN VALUES
-- STRING with all characters that were in file.
--
--*/
global function fast_file_read(sequence inFile)
	sequence buffer
	integer fi,len,o

	fi = open(inFile, "rb")
	if fi = -1 then
	    error ("Couldn't open file \"" & inFile & "\".")
	end if

	o = seek(fi,-1) 		-- go to end of input file
	len = where(fi) 		-- get length of input file in bytes
	o = seek(fi,0)			-- go back to beginning of input file
	buffer = repeat(0, len) -- initialize your buffer
	for i=1 to len do
	    buffer[i] = getc(fi)
	end for
	close(fi)

	return buffer
end function

--/*
-- write_file [Created on 21. December 2001, 15:54]
-- The 'write_file' procedure writes 'content' into file 'file_name'.
-- If file already exists it is deleted.
-- If it fails it dispalys error message.
--
-- PARAMETERS
-- 'file_name'
--    Name of the file to which to write.
-- 'content'
--    String to write into file.
--*/
global procedure write_file (STRING file_name, STRING content)
    --// File number.
    integer fn
    fn = open (file_name, "w")
    if fn = -1 then --// Couldn't open file.
        error ("Couldn't open file \"" & file_name & "\" for writing.")
        return
    end if
    puts (fn, content)
    close (fn)
end procedure

--/*
-- get_program_directory [Created on 30. September 2001, 15:48]
-- The 'get_program_directory' function returns directory where this program which is run is.
-- It gets it from parsing command line.
--
-- RETURN VALUES
-- Program directory, string. Like this: "C:\Directory"
--
--*/
global function get_program_directory ()
    --// Command line.
    sequence cmd
    --// Program path + name.
    STRING program_full_path
    --// Returned program directory
    STRING program_directory
    cmd = command_line ()
    if length  (cmd) >=  2 then --// Length of command line is long enough.
        program_full_path = cmd [2]
        program_directory = get_file_directory  (program_full_path)
    else                        --// Length of command line is too short.
        error ("Can't get program directory from command line.")
        program_directory = ""
    end if
    return program_directory
end function

--// By Brian Broker.
--// Replaces all occurances of substrings in string with new substring.
--// replace ("AA bb CC AA", "AA", "XX") returns "XX bb CC XX".
global function replace_str( sequence target, sequence old, sequence new )
  integer len, loc

  len = length( old )
  loc = match( old, target )

  if loc then
    return target[1..(loc - 1) ] &
           new &
           -- recursive call on remainder of string --
           replace_str( target[(loc + len)..length( target )], old, new )
  else
    return target
  end if

end function

--/*
-- replace_seq [Created on 23. June 2002, 17:13]
-- The 'replace_seq' function replaces all
-- elements of 'target' which are equal to 'new'
-- with 'old'.
-- It only replaces non-nested members.
--
-- PARAMETERS
-- 'target'
--    .
-- 'old'
--    .
-- 'new'
--    .
--
-- RETURN VALUES
-- sequence.
--
-- EXAMPLE
-- replace_seq ({1,2,3,1}, 1, 5) returns {5,2,3,5}.
--*/
global function replace_seq (sequence target, object old, object new)
    integer len, loc

    loc = find( old, target )

    if loc then
        target [loc] = new
        for i = loc + 1 to length (target) do
            if equal (target [i], old) then
                target [i] = new
            end if
        end for
    end if
    return target
end function

--/*
-- match_in_members [Created on 29. December 2001, 02:49]
-- The 'match_in_members' function performs 'match ()'
-- on every member of 's'.
--
-- PARAMETERS
-- 'find_me'
--    This searching.
-- 's'
--    On members in this sequence is 'match ()' used.
--
-- RETURN VALUES
-- If found it returns index of first member that contained 'find_me'.
-- 
-- If not found 0 is returned.
--*/
global function match_in_members (sequence find_me, sequence s)
    --// Index where in current member was 'find_me'.
    integer index
    for i = 1 to length (s) do
        index = match (find_me, s [i])
        if index != 0 then --// Bingo! Found!
            return i
        end if
    end for
    return 0
end function





--//--//--//--//--//--//--//--//--//-- Members retrival from sequences. --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//

global function get_first_member (sequence s)
    if length (s) >= 1 then
        return s [1]
    else
        error ("Length of sequence is 0, can't get first member.")
    end if
end function

global function get_last_member (sequence s)
    if length (s) != 0 then
        return s [length (s)]
    else
        error ("Length of sequence is 0, can't get last member.")
    end if
end function

global function get_member (sequence s, integer index)
    if index <= length (s) then
        return s [index]
    else
        error ("There is no element at that position in sequence.")
    end if
end function





--//--//--//--//--//--//--//--//--//-- Safe members retrival from sequences. --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//
--// Functions to safe retreive a meber from sequence.
--// If member is not found at the position then integer 'EMPTY' is returned.

global function get_first_member_safe (sequence s)
    if length (s) >= 1 then
        return s [1]
    else
        return types:EMPTY
    end if
end function

global function get_last_member_safe (sequence s)
    if length (s) != 0 then
        return s [length (s)]
    else
        return types:EMPTY
    end if
end function

global function get_member_safe (sequence s, integer index)
    if index <= length (s) then
        return s [index]
    else
        return types:EMPTY
    end if
end function

--/*
-- is_null [Created on 12. January 2002, 23:33]
-- The 'is_null' function finds out if an object is 
-- integer and 0.
--
-- PARAMETERS
-- 'o'
--    object to be checked.
--
-- RETURN VALUES
-- If object is integer 0 then true is returned.
-- Else false is returned.
--*/
global function is_null (object o)
    if integer (o) and o = 0 then
        return true
    else
        return false
    end if
end function

--/*
-- count_if [Created on 29. May 2002, 08:00]
-- The 'count_if' function returns integer how many times
-- is 'o' found in 's'. Looked are only non-nested members.
--
-- PARAMETERS
-- 's'
--    sequence searched thru.
-- 'o'
--    this element is being counted.
--
-- RETURN VALUES
-- Number of times 'o' is in 's'.
--*/
global function count_if (sequence s, object o)
    integer count
    count = 0 
    for i = 1 to length (s) do
        if equal (s [i], o) then
            count += 1
        end if
    end for
    return count
end function

--/*
-- convert_numbers_to_string [Created on 2. June 2002, 07:56]
-- The 'convert_numbers_to_string' function converts numbers in
-- sequence to one stering, separated by comma.
--
-- PARAMETERS
-- 'numbers'
--    Sequence with numbers.
--
-- RETURN VALUES
-- STRING, numbers separated by comma.
--*/
global function convert_numbers_to_string (sequence numbers)
    STRING res
    res = ""
    for i = 1 to length (numbers) do
        if integer (numbers [i]) then
            res &= sprintf ("%d,", numbers [i])
        elsif atom (numbers [i]) then
            res &= sprintf ("%.2f,", numbers [i])
        else
            assert (false) --// non-numbers not allowed in sequence
        end if
    end for
    return res [1 .. length (res) - 1] --// remove last comma
end function

--/*
-- convert_object_to_string [Created on 5. June 2002, 01:38]
-- The 'convert_object_to_string' function converts any Eu object into string.
--
-- PARAMETERS
-- 'o'
--    object.
--
-- RETURN VALUES
-- STRING.
--*/
global function convert_object_to_string (object o)
    if integer (o) then
        return sprintf ("%d", o)
    elsif atom (o) then
        return sprintf ("%f", o)
    elsif is_string (o) then
        return o
    else
        return print_format (o)
    end if
end function

--/*
-- count_atoms [Created on 5. June 2002, 19:00]
-- The 'count_atoms' function counts how many
-- atoms are in nested etc sequences. 
--
-- PARAMETERS
-- 'o'
--    object.
--
-- RETURN VALUES
-- Number of atoms in Euphoria object 'o'.
--*/
global function count_atoms (object o)
    integer num
    num = 0
    if not sequence (o) then    
        return 1
    else
        for i = 1 to length (o) do
            num += count_atoms (o [i])
        end for
    end if
    return num
end function

--/*
-- split_text_into_lines [Created on 9. June 2002, 00:46]
-- The 'split_text_into_lines' function split text into
-- lines of text.
--
-- PARAMETERS
-- 'text'
--    To be split.
--
-- RETURN VALUES
-- Sequence with strings.
--*/
global function split_text_into_lines (STRING text)
    sequence lines
    integer prev_nl, i
    lines = {}
    prev_nl = 0
    i = 1
    while i <= length (text) do
        if text [i] = '\n' then
            if i > 2 and text [i - 1] = 13 then
                lines = append (lines, text [prev_nl + 1 .. i - 2])
            else
                lines = append (lines, text [prev_nl + 1 .. i - 1])
            end if
            prev_nl = i
        end if
        i += 1
    end while
    if i > 2 and text [i - 1] = 13 then
        lines = append (lines, text [prev_nl + 1 .. i - 2])
    else
        lines = append (lines, text [prev_nl + 1 .. i - 1])
    end if
    return lines
end function

global function join_text_lines (sequence lines)
    STRING res
    res = ""
    for i = 1 to length (lines) do
        res &= lines [i] & "\n"
    end for
    res = res [1 .. length (res) - 1] --// remove last '\n'
    return res
end function

--/*
-- find_largest_member_seq [Created on 9. June 2002, 07:52]
-- The 'find_largest_member_seq' function finds 
-- non-nested member sequence which 
-- is largest.
--
-- PARAMETERS
-- 's'
--    Sequence with sequences.
--
-- RETURN VALUES
-- Index of sequence or 0 if no sequences.
--*/
global function find_largest_member_seq (sequence s)
    integer res
    res = 0
    for i = 1 to length (s) do
        if sequence (s [i]) then
            if res = 0 then
                res = i
            else
                if length (s [i]) > length (s [res]) then
                    res = i
                end if
            end if
        end if
    end for
    return res
end function

--// Same as builtin open(), if error it reports it to user.
global function open_safe (STRING fname, STRING mode)
	integer fn
	fn = open (fname, mode)
	if fn = -1 then
		error ("Couldn't open file " & fname & ".\n")
	end if
	return fn
end function

--/*
-- is_str_number [Created on 22. June 2002, 18:30]
-- The 'is_str_number' function returns true
-- if there are ony digit characters inside string,
-- dot is also allowed and # at first place.
-- "#1.234" returns true.
-- "12a3" return false.
--
-- PARAMETERS
-- 's'
--    .
--
-- RETURN VALUES
-- BOOL.
--*/
global function is_str_number (STRING s)
    char c
    for i = 1 to length (s) do
        c = s [i]
        if c < '0' or c > '9' and c != '.' and c != '-' then
            if c != '#' and i != 1 then
                return false
            end if
        end if
    end for
    return true
end function

--// Returns true if only digits (0-9) are in string.
global function are_only_digits (STRING s)
    for i = 1 to length (s) do
        if not isdigit (s [i]) then
            return false
        end if
    end for
    if length (s) = 0 then
        return false
    end if
    return true
end function 

--/*
-- any_too_high_numbers [Created on 24. June 2002, 03:50]
-- The 'any_too_high_numbers' function returns true if
-- there are any numbers in sequence which are too
-- high or too low than Euphoria atom can handle.
--
-- PARAMETERS
-- 's'
--    .
--
-- RETURN VALUES
-- true if not ok, false if ok.
--*/
global function any_too_high_numbers (sequence s)
    for i = 1 to length (s) do
        if s [i] >= HIGHEST_ATOM or s [i] <= LOWEST_ATOM then
            return true
        end if
    end for
    return false
end function

--/*
-- split [Created on 7. July 2002, 00:49]
-- The 'split' function splits strings into parts, 
-- split is at each place where character you specify is.
--
-- PARAMETERS
-- 's'
--    String to split.
-- 'split_char'
--    Character which if found in string it splits it.
--
-- EXAMPLE
-- parts = split ("c:\\my documents\\audio/video files\\", '\\')
-- parts is:
-- {
-- "c:",
-- "my documents",
-- "audio/video files"
-- ""
-- }
--
-- RETURN VALUES
-- Sequence with strings.
--*/
global function split (STRING s, CHAR split_char)
    integer i, prev_i
    sequence parts --// returned
    i = find (split_char, s)
    if i then
        parts = {s [1 .. i - 1]}
    else
        return {s}
    end if
    i += 1
    prev_i = i
    while i <= length (s) do
        if s [i] = split_char then
            parts = append (parts, s [prev_i .. i - 1])
            prev_i = i + 1
        end if
        i += 1
    end while
    parts = append (parts, s [prev_i .. length (s)])
    return parts
end function

--/*
-- trim_char_front [Created on 7. July 2002, 00:56]
-- The 'trim_char_front' function removes character at
-- front if it is equal to the one you specified.
--
-- PARAMETERS
-- 's'
--    .
-- 'c'
--    .
--
-- RETURN VALUES
-- String.
--*/
global function trim_char_front (STRING s, CHAR c)
    if s [1] = c then
        return s [2 .. length (s)]
    end if
end function

--/*
-- join_with [Created on 7. July 2002, 01:56]
-- The 'join_with' function joins sequence with strings
-- and puts string 'between' betwen them
--
-- PARAMETERS
-- 'strings'
--    .
-- 'between'
--    .
--
-- RETURN VALUES
-- String.
--*/
global function join_with (sequence strings, STRING between)
    STRING res --// returned
    res = ""
    for i = 1 to length (strings) - 1 do
        res &= strings [i] & between
    end for
    res &= strings [length (strings)]
    return res
end function

-- takes an number and makes a sequence with commas every 3 digits:
-- by Dan Moyer
global function PrettyNumbers(object aNumber)
sequence PrettyNumber, Temp, decPart
integer TriCounter, decPos
PrettyNumber = {}
Temp = {}
decPart = {}
decPos = 0


TriCounter = 0
Temp = sprint(aNumber)

decPos = find('.', Temp)
if decPos then
   decPart = Temp[decPos..length(Temp)]
   Temp = Temp[1..decPos -1]
end if

for n = length(Temp) to 1 by -1 do
PrettyNumber = prepend(PrettyNumber,Temp[n])
TriCounter += 1
if TriCounter = 3 and n != 1 then
PrettyNumber = prepend(PrettyNumber,',')
end if
if TriCounter = 3 then
TriCounter = 0
end if
end for

if not atom(decPart) then
  PrettyNumber = PrettyNumber & decPart
end if

return PrettyNumber
end function

--// By Kat.
global function commatize(sequence number)
sequence result, deci,int
result = ""

if match(".",number)
  then int = number[1..match(".",number)-1]
       deci = number[match(".",number)+1..length(number)]
  else int = number
       deci = ""
end if

while length(int) do
  if length(int) >= 3
    then result = int[length(int)-2..length(int)] & "," & result
         int = int[1..length(int)-3]
    else result = int & "," & result
         int = ""
  end if
end while
result = result[1..length(result)-1]
--// Start Tone Skoda
--// result = result & "." & deci
if length (deci) then
    result = result & "." & deci
end if
--// End Tone Skoda
if result[1] = ',' then result = result[2..length(result)] end if
return result
end function -- commatize(sequence number)

global function bytes_to_megabytes (atom bytes)
    return bytes / (1024 * 1024)
end function

--/*
-- trim_alphas [Created on 7. July 2002, 09:11]
-- The 'trim_alphas' function removes any alpha
-- characters from start and end of strings.
--
-- EXAMPLE:
-- "abc123zi" returns "123"
--
-- PARAMETERS
-- 's'
--    .
--
-- RETURN VALUES
-- String.
--*/
global function trim_alphas (STRING s)
    integer starti, endi
    if length (s) = 0 then
        return ""
    end if
    starti = 1
    while isalpha (s [starti]) and starti < length (s) do
        starti += 1
    end while
    endi = length (s)
    while isalpha (s [endi]) do
        endi -= 1
    end while
    return s [starti .. endi]
end function