





--// TSLibrary_Types.e
--// Tone koda <tone.skoda@siol.net>
--// Created on 6. October 2001.
--// This file contains global Euphoria types.





global constant false = 0, true = 1
--// Lowest/highest possible integer in Euphoria.
global constant LOWEST_INTEGER = -1073741824
global constant HIGHEST_INTEGER = 1073741823
--// Lowest/hieghest possible atom in Euporia.
global constant LOWEST_ATOM = -1e300
global constant HIGHEST_ATOM = +1e300
global constant LOWEST_ATOM_HALF = -1e300 / 2
global constant HIGHEST_ATOM_HALF = +1e300 / 2
--// Default, empty value.
global constant DEFAULT = LOWEST_INTEGER
global constant EMPTY = LOWEST_INTEGER
--// Something that is not defined, whatever you want to use it for...
global constant undefined = -1
--// Unneeded return values of functions go into this variable.
--// It should be used for nothing else.
global object Void
global constant TODO = 0





--/*
-- UNIQUE_STRICT [Created on 6. October 2001, 17:00]
-- The 'UNIQUE_STRICT' type is sequence which has members which are all different.
--
-- PARAMETERS
-- 's'
--    sequence.
--*/
global type UNIQUE_STRICT (sequence s)
    --// Current member, we're comparing each other member with this one.
    object cur_member

    --// Loop every member of 's', every time change 'cur_member'.
    for i = 1 to length (s) do
        cur_member = s [i]
        for j = 1 to length (s) do --// Loop every member of 's' again to compare ti against 'cur_member'.
            if i != j and equal (cur_member, s [j]) = true then --// Error: two member are equal.
                return false
            end if
        end for
    end for

    return true
end type

--/*
-- UNIQUE [Created on 6. October 2001, 17:00]
-- The 'UNIQUE' type is sequence which has members which are all different,
-- except if they are 0 or "" or {}.
--
-- PARAMETERS
-- 's'
--    sequence.
--*/
global type UNIQUE (sequence s)
    --// Current member, we're comparing each other member with this one.
    object cur_member

    --// Loop every member of 's', every time change 'cur_member'.
    for i = 1 to length (s) do
        cur_member = s [i]
        if equal (cur_member, {}) = false and equal (cur_member, 0) = false then --// Not 0 or {}.
            --// Loop every member of 's' again to compare it against 'cur_member'.
            for j = 1 to length (s) do
                if i != j then --// Not to compare member to itself.                
                        if equal (cur_member, s [j]) = true then --// Error: two members are equal.
                            return false
                        end if
                end if
            end for
        end if
    end for

    return true
end type

--/*
-- bool [Created on 6. October 2001, 17:10]
-- The 'bool' type: type should be 'false' or 'true'.
--
-- PARAMETERS
-- 'i'
--    integer.
--*/
global type bool (integer i)
    if i = true or i = false then
        return true
    else
        return false
    end if
end type

--/*
-- BOOL [Created on 27. December 2001, 20:14]
-- The 'BOOL' type.
--
-- PARAMETERS
-- 'i'
--    true or false.
--*/
global type BOOL (integer i)
    return bool (i)
end type

global function is_char (object i)
    if not integer (i) then
        return false
    end if
    return i >= -1 and i <= 255
end function

--/*
-- char [Created on 8. October 2001, 22:28]
-- The 'CHAR' type is character (letter) type.
--
-- PARAMETERS
-- 'i'
--    Integer.
--*/
global type CHAR (integer i)
    return is_char (i)
end type

--/*
-- char [Created on 8. October 2001, 22:28]
-- The 'char' type is character (letter) type.
--
-- PARAMETERS
-- 'i'
--    Integer.
--*/
global type char (integer i)
    return CHAR (i)
end type

--/*
-- STRING [Created on 6. October 2001, 17:13]
-- The 'STRING' type: every member should be a letter (integer in range 0-255).
--
-- PARAMETERS
-- 's'
--    sequence.
--*/
global type STRING (sequence s)
    --// Loop every member of 's'.
    for i = 1 to length (s) do
        if CHAR (s [i]) = false then --// Out of range.
            return false
        end if
    end for
    return true
end type

--/*
-- FLAT_SEQ [Created on 23. November 2001, 05:31]
-- The 'FLAT_SEQ' type: sequence which has only atoms.
--
-- PARAMETERS
-- 's'
--    sequence, flat.
--*/
global type FLAT_SEQ (sequence s)
    for i = 1 to length (s) do
        if not atom (s [i]) then --// Not!
            return false
        end if
    end for
    return true
end type

--/*
-- FLAT_SEQS [Created on 6. December 2001, 15:20]
-- The 'FLAT_SEQS' type:
-- Members are flat sequences.
--
-- PARAMETERS
-- 's'
--    Sequence.
--*/
global type FLAT_SEQS (sequence s)
    for i = 1 to length (s) do
        if FLAT_SEQ (s [i]) = false then --// Member is not flat sequence.
            return false
        end if
    end for
    return true
end type

--/*
-- ASC_SORTED [Created on 6. December 2001, 03:56]
-- The 'ASC_SORTED' type: sequence 's' must be sorted in ascending order.
--
-- PARAMETERS
-- 's'
--    ascending sorted sequence.
--*/
global type ASC_SORTED (sequence s)    
    for i = 1 to length (s) - 1 do
	if compare (s [i], s [i + 1]) > 0 then --// Next member is bigger than previous member.
	    return false
	end if
    end for
    return true
end type

--/*
-- ASC_SORTED_SEQS [Created on 6. December 2001, 16:29]
-- The 'ASC_SORTED_SEQS' type:
-- sequence which has sequences which should be sorted ascending (A-Z).
--
-- PARAMETERS
-- 's'
--    Sequence.
--*/
global type ASC_SORTED_SEQS (sequence s)
    for i = 1 to length (s) do
        if ASC_SORTED (s [i]) = false then --// Member is not ascending sorted sequence.
            return false
        end if
    end for
    return true
end type

--/*
-- BOOLEANS [Created on 12. December 2001, 00:14]
-- The 'BOOLEANS' type: all members should be integers which are 0 or 1 (false or true).
--
-- PARAMETERS
-- 's'
--    sequence with booleans.
--*/
global type BOOLEANS (sequence s)
    for i = 1 to length (s) do
        if bool (s [i]) = false then --// Current member is not bool, this type is not right.
            return false
        end if
    end for
    return true
end type

--// Rectangle type.

global constant
    RECT_LEFT = 1,
    RECT_TOP = 2,
    RECT_RIGHT = 3,
    RECT_BOTTOM = 4,
    RECT_SIZE = 4

global function new_rect ()
    return repeat (EMPTY, RECT_SIZE)
end function

global type RECT (sequence rect)
    if length (rect) != RECT_SIZE then
        return false
    end if
    if not atom (rect [RECT_LEFT]) then
        return false
    end if
    if not atom (rect [RECT_TOP]) then
        return false
    end if
    if not atom (rect [RECT_RIGHT]) then
        return false
    end if
    if not atom (rect [RECT_BOTTOM]) then
        return false
    end if
    return true
end type

--/*
-- COLOR [Created on 28. December 2001, 19:14]
-- The 'COLOR' type.
--
-- PARAMETERS
-- 'color'
--    RGB color (0, 0, 0) - (255, 255, 255) .
--*/
global type COLOR (integer color)
    if color < 0 or color > 16777215 then
        return false
    else
        return true
    end if
end type

global type USHORT (integer i)
    return i >= 0 and i < 65535
end type

global constant POINT_X = 1, POINT_Y = 2
global type POINT (sequence s)
    return length (s) = 2 and atom (s [1]) and atom (s [2])
end type

--/*
-- PERCENT [Created on 25. May 2002, 03:40]
-- The 'PERCENT' type:
-- Atom must be between 0 and 1 (inclusive).
--*/
global type PERCENT (atom a)
    return a >= 0 and a <= 1
end type