--------------------NOTICE-------------------------------
-- Software ID: tk_mem.e
-- Version:     0.54
-- Copyright:   (c) 2000 David Cuny, Derek Parnell
--              All rights reserved.
-- Licence:            
-- This software is provided 'as-is', without any express or implied warranty.
-- In no event will the authors be held liable for any damages arising from
-- the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictictions:
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software.
-- 2. If you use this software in a product, acknowedgement in the product's
--    documenation and binary are required.
-- 3. Altered source versions, and works substantially derived from the it,
--    must...
--   a) be plainly be marked as such,
--   b) not be misrepresented as the original software,
--   c) include this notice, unaltered.
--------------------End of NOTICE------------------------

-- The library was originally developed by David Cuny, and over time, a 
-- number of other people have made additions and corrections. Where
-- appropriate the work that other people have contributed is marked thus...

--     Begin <person's name>
--     End <person's name>

-----------------------------------------------------------------------------
-- Generic Tools
-- These are routines that I use often enough to wish they were part of
-- the standard Euphoria library

--/topic Memory Management routines
--/info
--Low-Level Memory management routines
without warning
without trace
include machine.e
include dll.e
include tk_misc.e

constant
HEAP_NO_SERIALIZE             = #00000001,
HEAP_GROWABLE                 = #00000002,
HEAP_GENERATE_EXCEPTIONS      = #00000004,
HEAP_ZERO_MEMORY              = #00000008,
HEAP_REALLOC_IN_PLACE_ONLY    = #00000010,
HEAP_TAIL_CHECKING_ENABLED    = #00000020,
HEAP_FREE_CHECKING_ENABLED    = #00000040,
HEAP_DISABLE_COALESCE_ON_FREE = #00000080,
HEAP_CREATE_ALIGN_16          = #00010000,
HEAP_CREATE_ENABLE_TRACING    = #00020000,
kernel32                      = open_dll( "kernel32.dll" ),
xHeapCreate  = define_c_func(kernel32, "HeapCreate", {C_LONG, C_LONG, C_LONG}, C_LONG),
xHeapDestroy = define_c_func(kernel32, "HeapDestroy", {C_LONG}, C_LONG),
xHeapAlloc   = define_c_func(kernel32, "HeapAlloc", {C_LONG, C_LONG, C_LONG}, C_LONG),
xHeapFree    = define_c_func(kernel32, "HeapFree", {C_LONG, C_LONG, C_LONG}, C_LONG),
xIsBadReadPtr= define_c_func(kernel32, "IsBadReadPtr", {C_LONG, C_LONG}, C_LONG)


integer  pAbortRtn  pAbortRtn  = -1	-- stores abort handler's routine id
-------------------------------
integer pAllocationCounter pAllocationCounter = 0

 -- store addresses of allocated strings.
sequence pAllocated pAllocated = {{},{}}
constant kOwner = 1
       , kAddr  = 2

integer  pAllotted  pAllotted  = 0	-- stores the accumulated size of a structure.
integer  pStringSet pStringSet = 0	-- The most recent string set number.

atom     pHeap      pHeap = 0

--/topic Memory Management Routines
--/func llSetAbort( i )
--/desc Sets the routine id of an Abort routine.
--/ret The previous value set.
--
-- Used to indicate if an error routine needs to be
-- called in the event of a catastophic error.
-- The error routine is assumed to be a procedure
-- that accepts a single sequence (typically an 
-- message string).
--
-- Example:
--
-- /code        
--      integer RtnID, OldID
--      RtnID = routine_id("abortErr")
--      OldID = llSetAbort(RtnID)
-- /endcode

global function llSetAbort(integer i)
-- Set the abort handler id. The routine must take two parameters,
-- a sequence (msg) and a integer (mode ==> 1=warning, 2=fatal)
    integer lOldRtn
    
    lOldRtn = pAbortRtn
    
    pAbortRtn = i
    
    return lOldRtn
end function

procedure myFree(atom pAddress)
    object VOID

    if pHeap = 0 then
        return
    end if

    VOID = c_func(xHeapFree,{ pHeap, 0, pAddress})
    pAllocationCounter -= 1

    if pAllocationCounter = 0 then
        VOID = c_func(xHeapDestroy,{ pHeap })
        pHeap = 0
    end if
            
end procedure

function myAllocate(integer pSize)
    atom lAddr
    if pHeap = 0 then
        pHeap = c_func(xHeapCreate,{0, 16000, 0})
        if pHeap = 0 then
            return 0
        end if
    end if

    lAddr = c_func(xHeapAlloc,{ pHeap, HEAP_ZERO_MEMORY, pSize})

    pAllocationCounter += 1

    return lAddr
end function

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- Memory Management memory management Routines
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
global constant
    Byte    = -1,
    Word    = -2,
    Integer = Word,
    Long    = -3,
    DWord   = Long,
    UInt    = Long,
    Ptr     = Long,
    Lpsz    = -4,
    Hndl    = -5,
    HndlAddr = -6,
    Strz    = -7
constant vSizeNames = {Byte, Word, Long, Lpsz, Hndl, HndlAddr, Strz}
constant vSizeLengs = {   1,    2,    4,    4,    4,        4,    1}

--/topic Memory Management Routines
--/func manage_mem( atom Owner, atom Address)
--/desc Records an acquired memory for garbage collection.
-- Normally this is handled automatically by /acquire_mem() but if you
-- are expected to manage some memory acquired by another means, such as
-- a Windows call or a 'C' routine, you can use this to record the memory for
-- subsequent release by /release_mem().
--
-- Example:
-- /code    
--      atom mset, pt, pstr
--
--      -- Establish a new memory set.
--      mset = /new_memset()
--      -- calls a routine which returns a structure address.
--      pt = c_func( xyz, {abc})
--      -- register this memory
--      manage_mem(mset, pt)
--      . . .
--      give all the memory area in 'mset' back
--      release_mem(mset)
-- /endcode

global procedure manage_mem( atom pOwner, atom pAddr )
    -- save location for garbage collection
    -- This also is used to move a block from one memset to another.
    integer lOwnerSub, lAddrSub

    -- Take the address out of any existing memset.
    lAddrSub = 0
    for i = 1 to length(pAllocated[kAddr]) do
        lAddrSub = find(pAddr, pAllocated[kAddr][i])
        if lAddrSub != 0 then
            pAllocated[kAddr][i] = removeIndex(lAddrSub, pAllocated[kAddr][i])
            exit
        end if
    end for
    
    -- Create a new memset if required.
    lOwnerSub = find(pOwner, pAllocated[kOwner])
    if lOwnerSub = 0 then
        pAllocated[kOwner] &= pOwner
        lOwnerSub = length(pAllocated[kOwner])
        pAllocated[kAddr] = append(pAllocated[kAddr], {})
    end if

    -- Store the address into the specified memset
    pAllocated[kAddr][lOwnerSub] &= pAddr
end procedure

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func acquire_mem( atom Owner, object structure )
--/desc Allocate memory for structure, and initialize to zero.
-- The memory allocated is linked to the /i Owner and all the
-- owner's memory can be released by one call. /n
-- If /i structure is a string, it is copied to the memory
-- location along with a zero byte.
--
-- If /i structure is an atom, it specifies that amount of memory
-- to acquire (a minimum of 4 bytes will always be acquired) and
-- the memory is set to all zeros.
--
--/ret Address of allocated memory.
--
-- Example:
-- /code    
--      atom mset, pt, pstr
--
--      -- Establish a new memory set.
--      mset = /new_memset()
--      -- get enough memory to hold a UInt datatype
--      xy = acquire_mem( UInt )
--      -- allocate a point structure
--      pt = acquire_mem( mset, SIZEOF_POINT )
--      -- copy a Euphoria string to a 'C' string area.
--      pstr = /acquire_mem( mset, "My String Data" )
--      . . .
--      give all the memory area in 'mset' back
--      release_mem(mset)
-- /endcode

global function acquire_mem( atom pOwner, object pData )

        -- allocate space for a structure (mininum of 4 bytes)
        -- and initialize to zero
        
    atom at

    if sequence(pData) then
        -- place string in memory
        at = myAllocate( 1 + length(pData) )
        if at != 0 then
            poke(at, pData)
            poke(at + length(pData) , 0)
        end if
    else
        -- Check for special datatype "names"
        if pData < 0 then
            pData = find(pData, vSizeNames)
            if pData != 0 then
                pData = vSizeLengs[pData]
            end if
        end if

        if pData < 4 then
          pData = 4
        end if

        at  = myAllocate( pData )

        if at != 0 then
            mem_set( at, 0, pData )
	    end if
    end if
    
    if at = 0 then
        if pAbortRtn >= 0
        then
            call_proc(pAbortRtn, { "Unable to allocate space.", 2 } )
        end if
    else
        manage_mem(pOwner, at)
    end if


    return at
        
end function

--/topic Memory Management Routines
--/proc release_mem( atom structure )
--/desc Returns the memory allocated by /acquire_mem() back to the system.
-- If /i structure is a memory set id, as returned by /new_memset(), then
-- all the memory owned in the memory set is returned and the memory set id
-- is released. That is, it cannot be reused. /n
-- If /i structure is a memory address returned by /acquire_mem(), then just
-- that memory is released. The memory set it belonged to is still usable.
--
-- Example:
-- /code    
--      atom mset, pt, pstr
--
--      -- Establish a new memory set.
--      mset = /new_memset()
--      -- get enough memory to hold a UInt datatype
--      xy = /acquire_mem( UInt )
--      -- allocate a point structure
--      pt = /acquire_mem( mset, SIZEOF_POINT )
--      -- copy a Euphoria string to a 'C' string area.
--      pstr = /acquire_mem( mset, "My String Data" )
--      . . .
--      give all the memory area in 'mset' back
--      release_mem(mset)
-- /endcode

global procedure release_mem( atom pData )
    integer lOwnerSub
    integer lAddrSub, lAddrList
    atom lAddrToRelease
    integer ls, ss, Phase1
    integer debugi
    sequence sets

    -- Check for "special" uninitialized memset value.
    if pData = -1 then
        return
    end if

    -- See if this is a memset owner.
    lOwnerSub = find(pData, pAllocated[kOwner])
    
    if lOwnerSub = 0 then
        -- If not a memset, see which memset it belongs to.
        lAddrSub = 0
        for i = 1 to length(pAllocated[kAddr]) do
            lAddrSub = find(pData, pAllocated[kAddr][i])
            if lAddrSub != 0 then
                lAddrList = i
                exit
            end if
        end for

        if lAddrSub = 0 then
            -- Not in any memset!
            if pAbortRtn >= 0 then
                call_proc(pAbortRtn, {"Trying to release unacquired memory", 2})
            end if
            
        else
            -- Remove it from the memset
            pAllocated[kAddr][lAddrList] = removeIndex(lAddrSub,
                                           pAllocated[kAddr][lAddrList])
        end if

        -- Give the memory back to the system
        myFree(pData)
        return
    end if

    --
    sets = {pData}
    ss   = 1

    -- Phase 1: Identified the complete hierarchy of owned memory sets.
    Phase1 = 1
    while Phase1 do
        ls = length(sets)
        for i = ss to ls do
            lOwnerSub = find(sets[i], pAllocated[kOwner])
            for j = 1 to length(pAllocated[kAddr][lOwnerSub]) do
                lAddrSub = find(pAllocated[kAddr][lOwnerSub][j],
                                pAllocated[kOwner])
                if lAddrSub != 0 then
                    sets &= pAllocated[kOwner][lAddrSub]
                end if
            end for
        end for
        ss = ls + 1
        Phase1 = (ls != length(sets))
    end while

    -- Phase 2: Free the memory addresses.
    for i = length(sets) to 1 by -1 do
        lOwnerSub = find(sets[i], pAllocated[kOwner])
        for j = 1 to length(pAllocated[kAddr][lOwnerSub]) do
            if pAllocated[kAddr][lOwnerSub][j] != 0 then
                myFree(pAllocated[kAddr][lOwnerSub][j])

            end if
        end for
    end for
    if sets[1] != 0 then
        myFree(sets[1])

    end if

    -- Phase 3: - clean up the allocation array
    for i = 1 to length(sets) do
        lOwnerSub = find(sets[i], pAllocated[kOwner])
        pAllocated[kAddr] = removeIndex(lOwnerSub, pAllocated[kAddr])
        pAllocated[kOwner] = removeIndex(lOwnerSub, pAllocated[kOwner])
        for j = 1 to length(pAllocated[kAddr]) do
            lOwnerSub = find(sets[i], pAllocated[kAddr][j])
            if lOwnerSub != 0 then
                pAllocated[kAddr][j] = removeIndex(lOwnerSub,
                                         pAllocated[kAddr][j])
                exit
            end if
        end for
    end for

end procedure

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/proc release_all_mem()
--/desc Free all the acquired memory
-- This gives back to the system, all the memory acquired by calling
-- /acquire_mem(). You must not use any previously acquired memory blocks
-- after this has been called. /n
--
-- /b"NOTE:" When using the Win32Lib library, it is /b not required to call
-- this function as that is done automatically when WinMain() completes.
--
-- /b"WARNING:" Calling this before WinMain() has ended will probably cause the
-- the /i Win32Lib routines to crash.
--
-- Example:
--
--/code
--      -- Return all the memory areas
--      release_all_mem()
--
--/endcode
global procedure release_all_mem()

    -- for each memory set...
    while length( pAllocated[kOwner] != 0 ) do
        -- return the acquired memory
        release_mem( pAllocated[kOwner][1] )
    end while

end procedure



-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func allot( object FldDefn )
--/desc Allocate space in structure for a new field.
--/ret SEQUENCE: Definition to allotted memory.
-- /i FldDefn is either a number of bytes to allocate, one of the predefined
-- datatypes (listed below), or a 2-element sequence containing a repeat count
-- and a datatype or length.
--
-- If a number of bytes is supplied, the field is aligned to the next 32-bit
-- boundry before allocation.
--
-- The returned allotment definition is used by /store and /fetch. It has the
-- following structure. /n
-- The definition has three items: /n
-- An /i offset, a /i datatype, and a /i"repeat length" /n
-- Allowable types are: /n
-- /li /b Byte: 8 bit value
-- /li /b Word: 16 bit value
-- /li /b Integer: 16 bit value, save as /b Word
-- /li /b Long: 32 bit value
-- /li /b DWord: 32 bit value, same as /b Long
-- /li /b Ptr: 32 bit value, same as /b Long
-- /li /b Hndl: 32 bit value, a pointer to a pointer
-- /li /b HndlAddr: 32 bit value,
-- /li /b Lpsz: Long pointer (32 bits) to zero delimited string
-- /li /b Strz: Fixed size buffer that holds a zero-delim string
-- 
-- Example:
--
-- /code
-- constant
--    msLeft           = allot( Long ),
--    msTop            = allot( Long ),
--    msRight          = allot( Long ),
--    msBottom         = allot( Long ),
--    msXYZ            = allot( {4, DWord} ),
--    msReserved       = allot( 5 ),
--    msName           = allot( Lpsz ),
--    msBuffer         = allot( {128, Strz} ),
--    SIZEOF_MYSTRUCT  = /allotted_size()
-- /endcode

global function allot( object pDataType )

    integer soFar, diff, size, i, lCnt

    if sequence(pDataType) then
        lCnt = pDataType[1]
        i =   pDataType[2]
    else
        lCnt = 1
        i   = pDataType
    end if
    
    -- save position    
    soFar = pAllotted

    -- if not a pre-defined type, make sure it gets a word boundary
    size = find(i, vSizeNames)
    if size = 0
    then
       if i > 0 then
          diff = remainder( soFar, 4 )
          if diff then
            -- word align
            soFar = soFar + 4 - diff
          end if

        -- size is actual size
        size = i
       end if
    else
        size = vSizeLengs[size]
    end if

    -- allot space
    pAllotted += (size * lCnt)

    
    -- return offset, data type and count
    return { soFar, i, lCnt }
    
end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func allotted_handle(hDefn)
--/desc Returns the address of the supplied handle, but with fetch instructions
-- to get the address from the handle, rather than the handle itself. /n
-- An empty sequence is returned if the parameter was invalid.
--/ret SEQUENCE: Handle's "offset" into a structure.
--
-- Example:
--
-- /code                                   
-- constant
--    hDemo            = /allot( Hndl ),
--    pDemo            = /allot_handle( hDemo ),
--    SIZEOF_DEMO      = /allotted_size()
--     . . .
--    x = allocate_struct(SIZEOF_DEMO)
--    initDEMO(x)
--
--    h = fetch(x, hDemo)
--    a = fetch(x, pDemo)
--    -- 'h' will contain the handle, and 'a' the address from the handle.
-- /endcode


global function allotted_handle(sequence pHandle)
   if length(pHandle) = 3
         and
      pHandle[2] = Hndl
   then
      return {pHandle[1], HndlAddr, pHandle[3]}
   else
      return {}
   end if
         
    
end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func allotted_size()
--/desc Return allocate size of structure, and reset for new structure.
--/ret INTEGER: Allotted size of structure.
--
-- Example:
--
-- /code                                   
-- constant
--    rectLeft            = /allot( Long ),
--    rectTop             = /allot( Long ),
--    rectRight           = /allot( Long ),
--    rectBottom          = /allot( Long ),
--    SIZEOF_RECT        = /allotted_size()
-- /endcode

global function allotted_size()

    -- returns allotted size, and clears size

    integer soFar
    
    soFar = pAllotted

    pAllotted = 0
    
    return soFar
    
end function
    

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/proc store( structure, field, value )
--/desc Store a value into a structure.
-- Type conversion is automatic. For example, if an /b Lpsz field is 
-- used, the value is automatically converted from a sequence to a 
-- C-style string, and the address of that string is stored in the 
-- structure.
--
-- Example:
--
--/code
--      -- allocate RECT structure, and populate it
--      atom rect                                  
--
--      -- allocate the structure
--      rect = /allocate_struct( SIZEOF_RECT )
--
--      -- store values into the structure
--      /store( rect, rectLeft,   x1 )
--      /store( rect, rectTop,    y1 )
--      /store( rect, rectRight,  x2 )
--      /store( rect, rectBottom, y2 )
--/endcode

global procedure store( atom struct, sequence s, object o )

    -- Store the data based on its type
    integer where, datatype, lCnt
    atom    at
    sequence bytes
    
    -- rest data type
    where    = s[1] + struct
    datatype = s[2]
    lCnt     = s[3]

    -- For sequences, make sure no more than lCnt elements are stored.
    if sequence(o) then
        if length(o) < lCnt then
            lCnt = length(o)
        end if
    end if
    
    -- read, based on datatype
    if      datatype = Byte then
        -- poke a byte
        if atom(o) then
            poke( where, o )
        else
            poke(where, o[1 .. lCnt])
        end if
                
    elsif   datatype = Word then
        -- poke a word
            bytes = int_to_bytes( o )
            poke( where, bytes[1 .. 2] )
                
    elsif   datatype = Long then
        -- poke long(s)
        if atom(o) then
            poke4( where, o )
        else
            poke4(where, o[1 .. lCnt])
        end if

    elsif   datatype = Lpsz then
        -- if atom, treat as long        
        if atom( o ) then
            poke4( where, o )
        else
            -- poke the address of the allotted string
            poke4( where, acquire_mem( struct, o  ) )
        end if
        
    elsif   datatype = Hndl then
        -- poke handle(s)
        if atom(o) then
            poke4( where, o )
        else
            poke4(where, o[1 .. lCnt])
        end if

    elsif   datatype = HndlAddr then
        -- poke a handle's address
        at = acquire_mem(struct, 4)
        poke4( at, o)
        poke4( where, at )

    elsif  datatype = Strz then
        -- poke a zero-terminated string
        bytes = o & 0
        if lCnt < s[3] then
            lCnt += 1
        end if
        poke(where, bytes[1 .. lCnt])
    else
        -- poke a string                        
        poke( where, o[1 .. lCnt] )
    end if

end procedure



-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func peek_string( address )
--/desc Get sequence from address holding C-style string.
--/ret SEQUENCE: containing the C-style string.
-- This is typically done automatically by the /fetch
-- function.
--
-- Example:
--
--/code
--      -- get a C-string from address
--      sequence s
--
--      s = /peek_string( address )
--/endcode
integer pPeekStringBufSize pPeekStringBufSize = 256
global function peek_string(atom a)

    integer i, l, sl
    sequence s

    -- Initialise
    s = {}
    sl = 0
    l = 0

    -- Only deal with non-zero addresses
    if a then
        -- pick up first byte
        i = peek(a)

        -- Repeat until we find a zero byte
        while i do
            -- increment string size so far
            l += 1

            -- if the string buffer size is not big enough
            -- then expand it.
            if sl < l then
               s &= repeat(0,pPeekStringBufSize)
               sl += pPeekStringBufSize
            end if

            -- put into the string buffer the byte
            s[l] = i

            -- Point to the next byte
            a += 1
            
            -- Pick up next byte
            i = peek(a)
        end while
        
    end if

    -- send back all the bytes found.
    return s[1 .. l]
end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func zStringLength( address )
--/desc Get the length of a C-style string.
--/ret INTEGER: The length of the string pointed to by /i address
--
-- Example:
--
--/code
--      -- get length of a C-string from address
--      integer len
--
--      len = zStringLength( address )
--/endcode
global function zStringLength(atom a)

    integer i, l, sl
    l = 0

    -- Only deal with non-zero addresses
    if a then
        -- pick up first byte
        i = peek(a)

        -- Repeat until we find a zero byte
        while i do
            -- increment string size so far
            l += 1
            -- Point to the next byte
            a += 1
            -- Pick up next byte
            i = peek(a)
        end while
        
    end if

    -- send back the size
    return l
end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/proc set_peek_string( integer newsize )
--/desc sets the size of the internal buffer used to convert 'C' strings into sequences.
-- The initial buffer is 256 bytes and should be adequate. The only effect this
-- has is on the speed of conversion. Larger values may speed things up, smaller
-- values may slow things down.
--
-- If /i newsize is 0, then 1 is used. If /i newsize is less than zero, then 256
-- (the default) is used.
-- Example:
--
--/code
--      -- Change buffer size to 500 bytes.
--      set_peek_string( 500 )
--/endcode
global procedure set_peek_string(integer newsize)
    if newsize < 1
    then
        if newsize = 0 then
            newsize = 1
        else
            newsize = 256
        end if
    end if
   
    pPeekStringBufSize = newsize
   
end procedure

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func peek_handle( handle )
--/desc Get address from a Handle.
--/ret ATOM: address.
-- This is typically done automatically by the /fetch
-- function.
--
-- Example:
--
--/code
--      -- get the address from handle
--      atom a, h
--
--      a = /peek_handle( h )
--/endcode


global function peek_handle(atom a)
    return peek4u(a)
end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func fetch( structure address, field )
--/desc Fetch field from structure.
--/ret OBJECT: Field from a structure.
-- Data conversion is automatic. For example, if the field is 
-- an /b Lpsz, a sequence containing the string will automatically 
-- be returned.
--
-- Example:
-- /code                                  
--      -- fetch the average character width from the text metrics structure
--      width = /fetch( tm, tmAveCharWidth )
-- /endcode

global function fetch( atom struct, sequence s )

    -- fetch the data based on the type
    integer size, char, cnt
    atom at

    -- address is struct + offset
    at = struct + s[1]
    -- get data size
    size    = s[2]
    -- get repeation
    cnt     = s[3]
    
    -- read, based on size    
    if      size = Byte then    
        -- return byte
        if cnt > 1 then
            return (peek( {at, cnt}) )
        else
            return peek( at )
        end if
    elsif   size = Word then
        if cnt = 1 then
            -- return word
            return bytes_to_int( peek({at, 2}) & {0, 0} )
        else
            s = {}
            for i = 1 to cnt do
                s &= bytes_to_int( peek({at, 2}) & {0, 0} )
                at += 2
            end for
            return s
        end if
        
    elsif   size = Long then
        if cnt = 1 then
            -- return long
            return peek4s( at )
        else
            s = {}
            for i = 1 to cnt do
                s &= peek4s(at )
                at += 4
            end for
            return s
        end if

    elsif   size = Lpsz then            
        -- get the pointer
        at = peek4u(at)     
        if not c_func(xIsBadReadPtr,{at,4})then
            -- return the string
            return peek_string( at )
        else
            return at
        end if

    elsif   size = Hndl then    
        if cnt = 1 then
            -- return handle
            return peek4u( at )
        else
            s = {}
            for i = 1 to cnt do
                s &= peek4u(at )
                at += 4
            end for
            return s
        end if

    elsif   size = HndlAddr then    
        -- return a handle's address
        at = peek4u( at )
        at = peek_handle(at)
        return at
    elsif size = Strz then

        return peek_string(at)
                
    else
        -- return the string                        
        return( peek( {at, size} ) )
        
    end if

end function

-----------------------------------------------------------------------------
--/topic Memory Management Routines
--/func address( structure address, field )
--/desc Get address of /i field in structure.
--/ret ATOM: Address of the field in the structure.
-- This is typically used if the structure contains an array.
--
-- In this snippet, the /b memBitmapInfo structure contains an array
-- of /b RGBQUAD colors. The array is populated with the values in 
-- the pal:
--
-- /code 
--    -- get the start of the rgbQuad array
--    rgbQuad = address( memBitmapInfo, bmiColors )
--
--    -- copy the pal to memory    
--    for i = 1 to colors do
--
--        -- store values
--        /store( rgbQuad, rgbRed,      pal[i][1] )
--        /store( rgbQuad, rgbGreen,    pal[i][2] )
--        /store( rgbQuad, rgbBlue,     pal[i][3] )
--        /store( rgbQuad, rgbReserved, 0 )
--        
--        -- move to next quad
--        rgbQuad += SIZEOF_RGBQUAD
--
--    end for
--
-- /endcode
global function address( atom addr, object offset )
    -- return address in structure  
    if atom( offset ) then
        return addr + offset
    elsif length(offset) = 3 then
        return addr + offset[1]
    else
        return 0
    end if
end function


--/topic Memory Management Routines
--/func new_memset( a )
--/desc Allocates a unique id for a memory set.
--/ret ATOM: An id for a new memory set (memset).
-- A memset id is actually a machine address of a 4-bytes location. You can
-- use this 4-byte area for anything you like, until you call /release_mem()
--
-- Example:
--
--/code
--      atom ss
--
--		ss = new_memset()
--      b = acquire_mem(ss, "All you need is love")
--		...
--      release_mem( ss )  -- Let go of set 'ss'
--/endcode
global function new_memset()
atom ms
	ms = acquire_mem( 0, UInt )
    return ms
end function

